/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once

#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <malloc.h>
#include <glad/glad.h>
#include <cglm/call.h>

/**
 * Contains information about the current render state, can be used for querying
 * how the renderer is currently set up.
 */
typedef struct {
  /** Resolution (in pixels) */
  int32_t width, height;
} render_t;

/**
 * Initialize the renderer.
 * 
 * @param name String of the windows' name.
 * @return Rendering Context information.
 */
render_t * renderInit(char *name);

/**
 * Render a single frame of the render loop. The renderer is not (currently)
 * responsible for render looping.
 * 
 * @param render The renderer to loop for.
 */
void renderFrame(render_t *render);

/**
 * Cleanup a render context.
 * 
 * @param render Renderer to cleanup.
 * @return True or False if Successful/Not.
 */
bool renderDispose(render_t *render);

/**
 * Set the internal display resolution.
 * 
 * @param render Renderer to set resolution for.
 * @param width Width (in pixels) of the viewport.
 * @param height Height (in pixels) of the viewport.
 */
void renderSetResolution(render_t *render, uint32_t width, uint32_t height);