/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <stdbool.h>
#include "../engine.h"
#include "../platform.h"

/** Information about the current game context. */
#ifndef GAMETYPE_T
  #define GAMETYPE_T void
#endif
typedef GAMETYPE_T game_t;

/**
 * Initialize the game context.
 * 
 * @return The game instance context.
 */
game_t * gameInit(platform_t *platform);

/**
 * Start the main game loop.
 * 
 * @param game The game to start the loop for.
 */
void gameUpdate(game_t *game);

/**
 * Cleanup a previously constructed.
 * @param game The game to cleanup.
 */
void gameDispose(game_t *game);

/**
 * Because games are anonymously typed we simply request that they allow other
 * parts of the software to access the engine directly.
 * 
 * @param game The game context
 * @return The engine context attached to the running game.
 */
engine_t * gameGetEngine(game_t *game);