/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"

input_t * inputInit(uint32_t inputBindCount) {
  uint32_t i;

  // Create the input manager
  input_t *input = malloc(sizeof(input_t));
  if(!input) return NULL;

  // Setup the bind lists
  input->inputBindCount = inputBindCount;
  input->bindMap = malloc(sizeof(list_t) * inputBindCount);
  for(i = 0; i < inputBindCount; i++) {
    input->bindMap[i] = listCreate();
  }

  // Create the current & previous input states
  input->current = malloc(sizeof(float) * inputBindCount);
  input->previous = malloc(sizeof(float) * inputBindCount);

  // Create the input actuate times array.
  input->times = malloc(sizeof(float) * inputBindCount);

  // Pass off the input manager
  return input;
}

void inputUpdate(input_t *input) {
  uint32_t i;

  // Flip the states to save memory.
  float * currentCurrent = input->current;
  input->current = input->previous;
  input->previous = currentCurrent;

  // Now look at each bind, get the state from the platform
  for(i = 0; i < input->inputBindCount; i++) {
    // For this bind, get the sources.
  }
}

void inputDispose(input_t *input) {
  uint32_t i;

  // Free up the bind lists
  for(i = 0; i < input->inputBindCount; i++) {
    listDispose(input->bindMap[i], false);
  }

  free(input->times);
  free(input->previous);
  free(input->current);
  free(input->bindMap);
  free(input);
}

void inputBind(input_t *input, inputbind_t bind, inputsource_t *source) {
  listAdd(input->bindMap[bind], source);
}

void inputUnbind(input_t *input, inputbind_t bind,
  inputsource_t *source
) {
  listRemove(input->bindMap[bind], source);
}

bool inputIsDown(input_t *input, inputbind_t binding) {
  return input->current[binding] != 0;
}

bool inputIsUp(input_t *input, inputbind_t binding) {
  return input->current[binding] == 0;
}

bool inputIsPressed(input_t *input, inputbind_t binding) {
  return (
    input->previous[binding] == 0 &&
    input->current[binding] != 0
  );
}

bool inputIsReleased(input_t *input, inputbind_t binding) {
  return (
    input->current[binding] == 0 &&
    input->previous[binding] != 0
  );
}

float inputGetAxis(input_t *input, inputbind_t binding) {
  return input->current[binding];
}

float inputGetFullAxis(input_t *input, inputbind_t positive,
  inputbind_t negative
) {
  return input->current[negative] + input->current[positive];
}

float inputGetAccuated(input_t *input, inputbind_t binding) {
  return input->times[binding];
}